
/**
* Maintains all logs generated by the uninstall process.
* @constructor
*/
function ExtUninstallLogger(){
	
	// Actions
	this.READ_ACTION		= 20;
	this.WRITE_ACTION		= 21;
	this.ADD_ACTION			= 22;
	this.REMOVE_ACTION		= 23;
	this.EDIT_ACTION		= 24;
	this.EXECUTE_ACTION		= 25;
	this.UNINSTALL_ACTION	= 26;
	
	// Status
	this.SUCCESS_STATUS	= 30;
	this.EMPTY_STATUS	= 31;
	this.FAIL_STATUS	= 32;
	this.WARN_STATUS	= 33;
	
	this.type		= "logger";
	this.logs 		= new Array();
	
	
	/**
	* Add log item to log tree.
	* This will add it to the trunk (top level) of the tree.
	* To nest a log underneath another, call addChild() on that log item
	* @param {ExtuninstallLogItem} oLogItem
	* @see ExtuninstallLogItem.addChild
	* @return ExtuninstallLogItem
	*/
	this.addLog = function(oLogItem){
		this.logs.push(oLogItem);
		return oLogItem;
	}
	
	/**
	* Returns true if any of the logs in the tree have an error
	* return boolean
	*/
	this.hasError = function(){ 
		for(var i = 0; i < this.logs.length; i++){
			if(this.logs[i].hasError()){
				return true;
			}
			
		}
		return false;
	}
	
	/**
	* Returns true if any of the logs in the tree have a warning
	* return boolean
	*/
	this.hasWarning = function(){ 
		for(var i = 0; i < this.logs.length; i++){
			if(this.logs[i].hasWarning()){
				return true;
			}
			
		}
		return false;
	}
}


/**
* Item containing information for a single log
* @constructor
*/
function ExtuninstallLogItem(iAction, iStatus, sMessage, iErrorCode, sError){
	this.type		= "logitem";
	this.action 	= iAction;
	this.status 	= iStatus;
	this.message	= sMessage;
	this.errorCode 	= iErrorCode;
	this.errorMsg 	= sError;
	this.children	= new Array();
	
	/**
	* Add a log item as a child to this one
	* @param {ExtuninstallLogItem} oLog
	* @return ExtuninstallLogItem
	*/
	this.addChild = function(oLog){
		this.children.push(oLog);
		return oLog;
	}
	
	/**
	* Returns true if any of the logs in the tree have an error
	* return boolean
	*/
	this.hasError = function(){ 
	
		var error = (new ExtUninstallLogger()).FAIL_STATUS;
		
		if(this.status == error){
			return true;
		}
		for(var i = 0; i < this.children.length; i++){
			if(this.children[i].hasError()){
				return true;
			}
			
		}
		return false;
	}
	
	/**
	* Returns true if any of the logs in the tree have a warning
	* return boolean
	*/
	this.hasWarning = function(){ 
	
		var warning = (new ExtUninstallLogger()).WARN_STATUS;
		
		if(this.status == warning){
			return true;
		}
		for(var i = 0; i < this.children.length; i++){
			if(this.children[i].hasWarning()){
				return true;
			}
			
		}
		return false;
	}
}
