/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Yahoo! Mail Notifier code.
 *
 * The Initial Developer of the Original Code is
 * Michael Kraft.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function ym_notifier() {
	
  this.loginURL  = "https://login.yahoo.com/config/login";
  this.verifyURL = "https://login.yahoo.com/config/login_verify2";
  this.checkURL1 = "http://us.update.companion.yahoo.com/slv/v4/not";
  this.checkURL2 = "http://uk.update.companion.yahoo.com/slv/v4/not";
  this.logoutURL = "http://login.yahoo.com/config/login?logout=1";
  this.mailURL   = "http://mail.yahoo.com";

  // preferences
  this.prefs = new ym_prefs();
  this.prefs.initPrefs();
  
  // current password for default user
  this.currentDefaultPassword = null;
  
  this.loginFailCount = 0;
  this.checkFailCount = 0;
  this.trueNewMail = false;
  this.newMailCount = 0;
  this.doNotCheck = true;   // windows default to not checking

  this.mouseEvtTime = 0;    // save time of click event.
  this.mouseId = null;      // handle of click setTimeOut
  this.timer = null;        // NsiTimer object
  this.sound = null;        // NsiSound object

  // 
  this.xmlHttpReq = null;   // XMLHttpRequest object

  // NsiObserver object
  this.observer = {
    observe : function(aSubject, aTopic, aData) {
      gYm_notifier.observe(aSubject, aTopic, aData);
    }
  }
}

/*
 * do stuff when new browser window opens 
 * use a settimeout to allow window to open if masterpassword is set
 */
ym_notifier.prototype.initialize = function() {
  if (document.getElementById("ym-notifier-statusbar-item")) 
    window.setTimeout("gYm_notifier.initialize2()", 5);
}

ym_notifier.prototype.initialize2 = function() {
  var prefs = this.prefs;
  var showStatusbar = (prefs.getPref(prefs.PREF_STATUSBAR) == prefs.CONST_ALWAYS);
  var myStatusbar = document.getElementById("ym-notifier-statusbar-item");
  if (myStatusbar) myStatusbar.setAttribute("collapsed", !showStatusbar);
    
  try {
    // Add observer for when logging in and out and for changing new mail indicator
    var observerService = Components.classes["@mozilla.org/observer-service;1"].
                          getService(Components.interfaces.nsIObserverService);
    observerService.addObserver(gYm_notifier.observer,"ym-gui-change",false);   
    prefs.addObserver(prefs.PREF_STATUSBAR, gYm_notifier.observer);
 
    // Is this the first browser window?
    var wm = Components.classes['@mozilla.org/appshell/window-mediator;1'].
             getService(Components.interfaces.nsIWindowMediator);
    var browsers=wm.getZOrderXULWindowEnumerator('navigator:browser',true);
    if (browsers.hasMoreElements()) { var browser=browsers.getNext(); }

    // This is the first window
    if (!browsers.hasMoreElements()) {
      this.ym_log("First window\n");
      	
      // allow this window to check
      this.doNotCheck = false;
      
      // initialize XMLHttpRequest Object
      this.xmlHttpReq = new XMLHttpRequest();
      	
      // listen for interval changes and start checking for email if set to do so
      prefs.addObserver(prefs.PREF_INTERVAL, gYm_notifier.observer);
        
      if (prefs.getPref(prefs.PREF_AUTO_CHECK)) {
      	var loginState = this.loginState();
        this.ym_log("loginState : " + loginState + "\n");
        if (loginState < 4) {
          this.doLogin();
        }
        else {
          this.startChecking();
        }
      }
    }
    else {
      this.ym_log("Not first window\n");

      var xulWindow = browsers.getNext(); 
      xulWindow = xulWindow.QueryInterface(Components.interfaces.nsIXULWindow);

      // set icons, menus and tooltips to the same as other windows
      var masterStatusbar = xulWindow.docShell.contentViewer.DOMDocument
                            .getElementById("ym-notifier-statusbar-item");

      // get new mail count from tooltip
      if (masterStatusbar) {
      	var messageCount =  masterStatusbar.getAttribute("label");
      	if (messageCount) this.updateGUI(messageCount);
      	else this.updateGUI("0");
      }
    }
  }
  catch (ex) {
    dump(ex);
    this.ym_logError(ex, 1);
  }
}
   
/* 
 * do stuff when browser window closes
 */
ym_notifier.prototype.uninitialize = function() {
  try {
    gYm_notifier.ym_log("Uninitializing\n");
    // Remove observers
    var observerService = Components.classes["@mozilla.org/observer-service;1"].
                          getService(Components.interfaces.nsIObserverService);
    observerService.removeObserver(gYm_notifier.observer,"ym-gui-change");   
    gYm_notifier.prefs.removeObserver(gYm_notifier.prefs.PREF_STATUSBAR, gYm_notifier.observer);

    // Find another window to take over polling job
    var newYmNotifier = null;
    var wm = Components.classes['@mozilla.org/appshell/window-mediator;1'].
             getService(Components.interfaces.nsIWindowMediator);
    var browsers=wm.getZOrderXULWindowEnumerator('navigator:browser',true);
    if(browsers.hasMoreElements()) { 
      var xulWindow = browsers.getNext();
      xulWindow = xulWindow.QueryInterface(Components.interfaces.nsIXULWindow);
      var requestor = xulWindow.docShell.QueryInterface(Components.interfaces.nsIInterfaceRequestor);
      var win = requestor.getInterface(Components.interfaces.nsIDOMWindow);
      newYmNotifier = win.gYm_notifier;
      gYm_notifier.prefs.addObserver(gYm_notifier.prefs.PREF_INTERVAL, newYmNotifier.observer);
      
      // Allow new window to check
      newYmNotifier.doNotCheck = false;
      
      // Copy current default password to new window
      newYmNotifier.currentDefaultPassword = gYm_notifier.currentDefaultPassword;
    }

    // Is this window polling, if so get another window to do it
    // A component would be better, do that later
    if (gYm_notifier.timer) {
      gYm_notifier.prefs.removeObserver(gYm_notifier.prefs.PREF_INTERVAL, gYm_notifier.observer);
      gYm_notifier.stopChecking();
      if (newYmNotifier) newYmNotifier.startChecking();
    } 
      
  }
  catch (ex) {
    gYm_notifier.ym_logError(ex, 1);
  }
  finally {
    // remove instance to ourselves
    gYm_notifier = null;
  }
}
   
/*
 * start checking for new mail
 */
ym_notifier.prototype.startChecking = function() {
  this.ym_log("begin startChecking\n");
	
  if (this.doNotCheck) {
    this.ym_log("Window not allowed to check\n");
    return;
  }

  if (this.timer) {
    this.timer.cancel();
    this.ym_log("restarting timer\n");
  }
  // this needs to be done even if the timer is cancel because of bug 325418 otherwise time will be off
  // if user changes interval
  // https://bugzilla.mozilla.org/show_bug.cgi?id=325418
  this.timer = Components.classes["@mozilla.org/timer;1"].
               createInstance(Components.interfaces.nsITimer);

  this.timer.init(gYm_notifier.observer, this.prefs.getPref(this.prefs.PREF_INTERVAL) * 60000, 
                  Components.interfaces.nsITimer.TYPE_REPEATING_PRECISE);

  this.checkMail();

  this.ym_log("leave startChecking\n");
}

/*
 * stop checking for new mail
 */
ym_notifier.prototype.stopChecking = function() {
  this.ym_log("stopChecking\n");
  if (this.timer) {
    this.ym_log("timer canceled");
    this.timer.cancel();
    this.timer = null;
  }
}

/*
 * log out of Yahoo! by deleting cookie
 */
ym_notifier.prototype.logout = function(username) {
  this.ym_log("logging out\n");
  try {
    // logout by deleting login cookie
    var cookMan = Components.classes["@mozilla.org/cookieService;1"].
                  getService(Components.interfaces.nsICookieManager);
    cookMan.remove(".yahoo.com", "Y", "/", false);

    // if default user forget default password
    if (username = this.prefs.getPref(this.prefs.PREF_DEFAULT_USER))
      this.currentDefaultPassword = null;

    // clear stored cookie
    this.prefs.saveUserCookie("", username);

    // notify all windows of logout
    var observerService = Components.classes["@mozilla.org/observer-service;1"].
                          getService(Components.interfaces.nsIObserverService);
    observerService.notifyObservers(null,"ym-gui-change","logout " + username);
  }
  catch (ex) {
    this.ym_logError(ex, 1);
  }
}

/*
 * Manual check for email or login if not already logged in
 */
ym_notifier.prototype.manualCheckMail = function () {
  var loginState = gYm_notifier.loginState();
  this.ym_log("manulCheckMail\nloginState : " + loginState + "\n");
  if (loginState < 4) {
    gYm_notifier.doLogin()
  }
  else {
    gYm_notifier.checkMail();
  }
}

/* 
 * Login (if necessary) and begin polling
 */
ym_notifier.prototype.doLogin = function() {
  this.ym_log("doLogin\n");

  // A better way would be to observe changes to the Yahoo cookies and automatically clear
  // the stored cookie if user logs out at web site, but this works for now.  A minor problem
  // is that Yahoo will keep asking to verify the password even though we are logged in.  
  // Observing when the new cookie is stored in the browser and storing it into the user 
  // would lessen this problem.  Really we should "verify" the login at least once every
  // 24 hours.
  // The current work around is to "forget" the login cookie if over 12 hours has passed
  // since the user last actually logged (didn't use cookie)

  var username = this.prefs.getPref(this.prefs.PREF_DEFAULT_USER);
  if (username) {
    var userInfo = this.prefs.loadUser(username);
    if (userInfo) {
      var password = userInfo.password; 
      var thecookie = userInfo.cookie;
      var useCookie = userInfo.useCookie;
      delete userInfo; 

      if (useCookie && thecookie && this.getCookie('.yahoo.com', 'T')) {
        this.loginWithCookie(username, thecookie);
        return;
      }
      else if (password) {
        this.login(username, userInfo.password);
        return;
      }
      // temporarily stored password for default user
      else if (this.currentDefaultPassword) {
        this.login(username, this.currentDefaultPassword);
        return;
      }
    }
  }
  // either no user or no password so open login window
  this.openLoginWindow();
}

/*
 * log in to Yahoo! using stored cookie info
 */
ym_notifier.prototype.loginWithCookie = function(username, cookieValue) {

  this.ym_log("Logging in with cookie " + cookieValue + "\n");

  var ioService = Components.classes['@mozilla.org/network/io-service;1'].
                  getService(Components.interfaces.nsIIOService);
  var cookieService = Components.classes["@mozilla.org/cookieService;1"].
                      getService(Components.interfaces.nsICookieService);
  var observerService = Components.classes["@mozilla.org/observer-service;1"].
                        getService(Components.interfaces.nsIObserverService);
  
  var uri = ioService.newURI("http://yahoo.com", null, null);
  var cookieString = "Y=" + cookieValue + ";domain=.yahoo.com;path=/;expires=Thursday, April 15, 2010 3:57:58 PM;"
  cookieService.setCookieString(uri, null, cookieString, null);

  observerService.notifyObservers(null,"ym-gui-change","login " + username);
  gYm_notifier.loginFail = false;
}

/* 
 * log in to Yahoo! if necessary
 */
ym_notifier.prototype.login = function(username, pword, persist) {
  var loginState = this.loginState(username);
  var theURL = this.loginURL;
  var persistent = "&.persistent=Y";

  if (persist==false || loginState == 2 || loginState == 4) persistent = "";

  this.ym_log("Logging in as " + username + " with persist = " + persist + "\n");

  if (loginState == 1) {
    this.ym_log("Logging out first because logged in as wrong user.\n");
    this.logout(username);
  }
  else if (loginState > 1) {
    this.ym_log("User was already logged in, verifying login.\n");
    theURL = this.verifyURL;
  }

  try {
    this.xmlHttpReq.open("POST", theURL, true);
    this.xmlHttpReq.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
    this.xmlHttpReq.onerror = function() {
      gYm_notifier.ym_log("login Onerror fired.");
      gYm_notifier.xmlHttpReq.abort();
      var observerService = Components.classes["@mozilla.org/observer-service;1"].
                            getService(Components.interfaces.nsIObserverService);
      observerService.notifyObservers(null,"ym-gui-change","login-error " + username);
    }
    this.xmlHttpReq.onload = function() {
      if (gYm_notifier.xmlHttpReq.readyState==4) {
        if (gYm_notifier.xmlHttpReq.status == 200) {
          gYm_notifier.loginFailCount = 0;
          var observerService = Components.classes["@mozilla.org/observer-service;1"].
                                getService(Components.interfaces.nsIObserverService);
          if (gYm_notifier.xmlHttpReq.responseText.indexOf('window.location.replace') != -1) {
            // notify all windows of login
            observerService.notifyObservers(null,"ym-gui-change","login " + username);
            gYm_notifier.loginFail = false;
            // save password if this was default user
            if (username = gYm_notifier.prefs.getPref(gYm_notifier.prefs.PREF_DEFAULT_USER))
              gYm_notifier.currentDefaultPassword = pword;
          }
          else {
            alert ('"' + username + '" - ' + gYm_notifier.getString("baduser"));
            gYm_notifier.loginFail = true;
            observerService.notifyObservers(null,"ym-gui-change","logout " + username);
          }
        }
        else {
          gYm_notifier.loginFailCount = gYm_notifier.loginFailCount + 1;
          if (gYm_notifier.loginFailCount > 3) {
            gYm_notifier.loginFailCount = 0;
            alert('"' + username + '" - ' + gYm_notifier.getString("loginfail"));
            gYm_notifier.xmlHttpReq.abort();
          }
          else {
            gYm_notifier.login(username, pword);
          }
        }
      } 
    }
    this.xmlHttpReq.send('.tries=1&hasMsgr=0&.v=0&.chkP=Y&.done=http://mail.yahoo.com' + 
                          persistent + '&login=' + encodeURIComponent(username) + '&passwd=' + 
                          encodeURIComponent(pword));
  }
  catch (ex)
  {
    this.ym_logError(ex, 1);
  }
}
   
/*
 * check for new mail
 */   
ym_notifier.prototype.checkMail = function() {

  // Check if its time to relogin to Yahoo before checking for mail
  var userInfo = gYm_notifier.prefs.loadUser("", false);
  var username = null;
  if (userInfo) {
     var useCookie = userInfo.useCookie;
     username = userInfo.username;
     delete userInfo;
     if (!useCookie) {
       this.ym_log('Time to relogin to Yahoo\n')
       this.doLogin();
       return;
     }
  }
  	
  this.ym_log('Checking mail\n')
  var req = new XMLHttpRequest();
  var checkURL = (this.prefs.getPref(this.prefs.PREF_ALTERNATE_SERVER)) ? this.checkURL2 : this.checkURL1;
  this.xmlHttpReq.open("GET", checkURL, true);
  this.xmlHttpReq.onerror = function() {
    // error occur, try again.
    gYm_notifier.ym_log("checkmail Onerror fired");
    gYm_notifier.xmlHttpReq.abort();
    gYm_notifier.checkFailCount = gYm_notifier.checkFailCount + 1;
    if (gYm_notifier.checkFailCount > 3) {

      var observerService = Components.classes["@mozilla.org/observer-service;1"].
                            getService(Components.interfaces.nsIObserverService);
      observerService.notifyObservers(null,"ym-gui-change","check-error");
    }
  }
  this.xmlHttpReq.onload = function() {
    gYm_notifier.checkFailCount = 0;
    gYm_notifier.ym_log('request status (' + req.status + ')\n');
    if ((gYm_notifier.xmlHttpReq.readyState==4) && (gYm_notifier.xmlHttpReq.status == 200)) {
      var response = gYm_notifier.xmlHttpReq.responseText;
      var re = new RegExp(/.*\d+.*\./g);
      var result = re.exec(response);  
      if ((result) && (result[0].length > 4)) result = result[0].substring(2,result[0].length-2);

      try {
        if (!result) {
          gYm_notifier.ym_log("no mail\n");
          gYm_notifier.alertUser(0);
        }
        else {
          gYm_notifier.ym_log("new mail\n");
          var nbMessages = gYm_notifier.parseNumbFromString(result);
          
          // Alert the user if needed
          gYm_notifier.alertUser(nbMessages);
   
          // Find mailbox URL
          re = new RegExp(/http:\/\/[(a-z)+\.]+\.yahoo[\.(a-z)+]+/g);
          result = re.exec(response);
          gYm_notifier.ym_log(result + "\n");

          //  Yahoo doesn't set this correctly for all countries so allow user to override
          var userInfo = gYm_notifier.prefs.loadUser("", false);
          if (userInfo && !userInfo.overrideURL) {
            delete userInfo;
            if (!result) result = gYm_notifier.mailURL;
            gYm_notifier.prefs.saveUserMailURL(result.toString());
          }
          userInfo = null;
        }
      }
      catch(ex) {
        gYm_notifier.ym_logError(ex, 1);
      }
    }
  }
  this.xmlHttpReq.send(null);
}

// Cookie functions to determine if user is logged in or not
// This will not tell what username user is logged in under though
//
// Returns:
//   0 if no user logged in
//   1 if logged in as different user
//   2 if logged in as correct user for browser session, but we want to verify login
//   3 if permanently logged in logged in as correct user, but we want to verify login 
//   4 if logged in as correct user for browser session
//   5 if permanently logged in logged in as correct user 
//
ym_notifier.prototype.loginState = function(username) {

  // 'p' and 'l' will never change for a username, 'n' will change if password changes
	
  var cookie = this.getCookie('.yahoo.com', 'Y');
  if (!cookie) return 0;

  var value = cookie.value;
  if (!value) return 0;

  var cookieData = this.decodeQueryString(value);
  if (!cookieData) return 0;
  if (!cookieData['l'] || cookieData['l'] == '') return 0;

  var rc;
  var userInfo = this.prefs.loadUser(username, false);
  if (userInfo && userInfo.cookie) {
    var userCookie = this.decodeQueryString(userInfo.cookie)
    if (userCookie && (cookieData['l'] == userCookie['l']) && (cookieData['p'] == userCookie['p'])) {
      if (cookieData['n'] == userCookie['n']) {
        if (userInfo.useCookie) {
          if (cookieData['np'] == 1) rc = 4;
          else rc = 5;
        }
        else {
          if (cookieData['np'] == 1) rc = 2;
          else rc = 3;
        }
      }
      else {
      	// n's aren't equal at this point which means password was changed so clear stored cookie
      	// and return 1
        this.ym_log(username + " changed password, clearing stored cookie.\n");
        this.prefs.saveUserCookie("", username);
      	rc = 1;
      }
    }
    else rc = 1;
  }
  else rc = 1;
  	
  if (userInfo) delete userInfo;
  return rc;
}

ym_notifier.prototype.decodeQueryString = function(qs) {
  qs = qs.replace(/%26/g, '&');
  qs = qs.replace(/%3D/g, '=');
  qs = qs.replace(/%25/g, '%');
  qs = qs.replace(/%3F/g, '?');
  var queryHash = new Array();
  var pairs = qs.split('&');
  for(var i = 0; i < pairs.length; i++) {
    var split = pairs[i].split('=');
    var key = split.shift();
    var val = split.join('=');
    queryHash[key] = val;
  }
  return queryHash;
}

ym_notifier.prototype.getCookie = function(domain, name) {
  var cmClass = "@mozilla.org/cookiemanager;1";
  var cms = Components.classes[cmClass].getService();
  var cm = cms.QueryInterface(Components.interfaces.nsICookieManager);
  var enumerator = cm.enumerator;
  var cIface = Components.interfaces.nsICookie;
  var domainRegex = new RegExp(domain.replace('\.', '\\.')+'$');
  while (enumerator.hasMoreElements()) {
    var nextCookie = enumerator.getNext().QueryInterface(cIface);
    if (nextCookie.host.match(domainRegex)) {
      if (nextCookie.name == name) return nextCookie;
    }
  }
  return null;
}

/*
 * alert the user if needed depending on the preferences
 */
ym_notifier.prototype.alertUser = function(nbMessages) {
  var prefs = this.prefs;

  var observerService = Components.classes["@mozilla.org/observer-service;1"].
                        getService(Components.interfaces.nsIObserverService);
  observerService.notifyObservers(null,"ym-gui-change",nbMessages);

  if (nbMessages > 0) {
   	
    // Check to see if new mail count has changed to see if there really is new mail
    if (this.newMailCount != nbMessages) this.trueNewMail = true;
    else this.trueNewMail = false;
    this.newMailCount = nbMessages;
   	
    if ((prefs.getPref(prefs.PREF_SOUND) == prefs.CONST_REPEAT) ||
        ((prefs.getPref(prefs.PREF_SOUND) == prefs.CONST_ONCE) && this.trueNewMail)) {
      var soundUrl = prefs.getPref(prefs.PREF_SOUNDFILE);

      if ((soundUrl) && (soundUrl != "")) {
        try {
          var FileFactory = new Components.Constructor('@mozilla.org/file/local;1', 'nsILocalFile', 
                                                        'initWithPath');   	
          var soundFile = new FileFactory(soundUrl);
          var nsIIOService = Components.classes['@mozilla.org/network/io-service;1'].
                             getService(Components.interfaces.nsIIOService);
          var soundURI =  nsIIOService.newFileURI(soundFile,null,null);
          if (!this.sound) {
            this.sound = Components.classes["@mozilla.org/sound;1"].
                        createInstance(Components.interfaces.nsISound);
          }
          // store sound instance in main object since sound will stop playing
          // if instance is destroyed.
          this.sound.play(soundURI);
        } 
        catch(ex) { 
          this.ym_logError(ex, 1);
        }
      }
    }

    var titleText = this.getString("new") + " Yahoo! Mail (" + prefs.getPref(prefs.PREF_DEFAULT_USER) + ")";
    var textStr = this.getString("youhave") +" " + nbMessages + " ";
    if (nbMessages == 1) textStr = textStr + this.getString("message");
    else textStr = textStr + this.getString("messages");

    if ((prefs.getPref(prefs.PREF_SHOW_ALERTS) == prefs.CONST_REPEAT) ||
        ((prefs.getPref(prefs.PREF_SHOW_ALERTS) == prefs.CONST_ONCE) && this.trueNewMail)) {
      try {
        var alerts = Components.classes["@mozilla.org/alerts-service;1"].
                     getService(Components.interfaces.nsIAlertsService);
        alerts.showAlertNotification("chrome://ym_notifier/skin/newmail.png", 
                                     titleText, textStr, true, "", this);
      } 
      catch(ex) { 
        this.ym_logError(ex, 1);
      }
    }

    if ((prefs.getPref(prefs.PREF_SHOW_POPUP) == prefs.CONST_REPEAT) ||
        ((prefs.getPref(prefs.PREF_SHOW_POPUP) == prefs.CONST_ONCE) && this.trueNewMail)) {
      alert (titleText + "\n" + textStr);
    }
  }
  else {
    gYm_notifier.trueNewMail = false;
    gYm_notifier.newMailCount = 0;
  }
}

/*
 * Observer functions
 */  
ym_notifier.prototype.observe = function(aSubject, aTopic, aData) {
	
  this.ym_log("Observer callback - aSubject : " + aSubject + ", aTopic : " +
              aTopic + ", aData : " + aData + "\n");
	            
  switch (aTopic) {
    // alert went away by itself
    case "alertfinished":
      break;

    // user clicked alert                   
    case "alertclickcallback":
      this.clickAlert();
      break;

    // user logged in or out or change of number of new messages
    case "ym-gui-change":
      this.updateGUI(aData);
      break;

    // preferences changed
    case "nsPref:changed":

      switch(aData) {

        // checking interval change
        case this.prefs.PREF_INTERVAL: 
        
          if (this.timer || this.prefs.getPref(this.prefs.PREF_AUTO_CHECK))
            this.startChecking();
          break;
        
        case this.prefs.PREF_STATUSBAR:
        
          var prefs = gYm_notifier.prefs;
          var myStatusbar = document.getElementById("ym-notifier-statusbar-item");
          if (myStatusbar) {
            var showStatusbar = ((prefs.getPref(prefs.PREF_STATUSBAR) == prefs.CONST_ALWAYS) ||
                                 ((prefs.getPref(prefs.PREF_STATUSBAR) == prefs.CONST_ON_NEW_MAIL) && 
                                  (myStatusbar.getAttribute("newMail") == true)));
            myStatusbar.setAttribute("collapsed", !showStatusbar);
          }
          break;
      }
      break;

    // timer periodic call
    case "timer-callback":

      this.checkMail();
      break;
  }
}

// nsIAlertsService - nsIAlertListener was deprecated and replaced with nsIObserver
//                    This is to maintain 1.7 compatability.
ym_notifier.prototype.onAlertFinished = function(aCookie) {}
ym_notifier.prototype.onAlertClickCallback = function(aCookie) {
   this.clickAlert();
}

/*
 * Update statusbar and toolbar
 */
ym_notifier.prototype.updateGUI = function(aData) {
  var myStatusbar = document.getElementById("ym-notifier-statusbar-item");
  var myToolbar = document.getElementById("ym-notifier-toolbar-item");
  var myLogout = document.getElementById("ym-notifier-context-menu-logout");
  var myMenu = document.getElementById("ym-notifier-context");
  var loginState = this.loginState();

  aData = aData.split(" ");
  
  var checkerror = (aData[0] == "check-error");
  var loginerror = (aData[0] == "login-error");
  var justloggedin = ((aData[0] == "login") && (loginState > 0));
  var loggedin = ((aData[0] != "logout") && ((loginState > 1) || justloggedin));
  var mailChange = ((aData[0] != "login") && (aData[0] != "logout"));
  var newMail = (aData[0] > 0); 
  var myTooltip;
  
  this.ym_log("loginState : " + loginState + ", checkerror : " + checkerror +
              ", loggedin : " + loggedin + ", justloggedin : " + justloggedin +
              ", mailChange : " + mailChange + ", newMail : " + newMail + 
              ", aData[0] : " + aData[0] + ", aData[1] : " + aData[1] + "\n");


  // errors take highest preference
  if (loginerror || checkerror) {
    if (loginerror) myTooltip = aData[1] + " " + this.getString("loginerror");
    else myTooltip = this.getString("checkerror");
    newMail = false;
    aData[0] = "";
    mailChange = true;
  }
  // logging out takes preference over everything except errors
  else if (!loggedin) {
    this.stopChecking();
    myTooltip = this.getString("nologin");
    newMail = false;
    mailChange = true;
    aData[0] = "";
  }
  else {
    if (justloggedin) {
      // store updated login and name part of the login cookie
      var cookie = this.getCookie('.yahoo.com', 'Y');
      this.prefs.saveUserCookie(cookie.value);
//      var cookieData = this.decodeQueryString(cookie.value);
//      this.prefs.saveUserCookie('l=' + cookieData['l'] + "&n=" + cookieData['n'] +
//                                '&p=' + cookieData['p'], aData[1]);

      // only start checking on login if not already checking
      if (!this.timer) this.startChecking();
    }

    if (newMail) {
      myTooltip = this.getString("youhave") + " " + aData[0] + " ";
      if (aData[0] == 1) myTooltip = myTooltip + this.getString("message");
      else myTooltip = myTooltip + this.getString("messages");
    }
    else {
      myTooltip = this.getString("nomail");
    }  
  }
      
  if (myStatusbar) {
    myStatusbar.setAttribute("logged-in", loggedin);
    myStatusbar.setAttribute("error", (checkerror || loginerror) );
    if (mailChange) {
      myStatusbar.setAttribute("tooltiptext", myTooltip);
      myStatusbar.setAttribute("newMail", newMail);
      myStatusbar.setAttribute("label", aData[0]);
      
      // show or hide status bar if setting is set to only show on new mail
      if (this.prefs.getPref(this.prefs.PREF_STATUSBAR) == this.prefs.CONST_ON_NEW_MAIL)
        myStatusbar.setAttribute("collapsed", !newMail);
    }
  }
  if (myToolbar) {
    myToolbar.setAttribute("logged-in", loggedin);
    myToolbar.setAttribute("error", checkerror);
    if (mailChange) {
      myToolbar.setAttribute("tooltiptext", myTooltip);
      myToolbar.setAttribute("newMail", newMail);
      myToolbar.setAttribute("unread", aData[0]);
    }
  }
  if (myLogout) myLogout.setAttribute("disabled", !loggedin);
  
  // hide the context popup menu if it is displayed
  if (myMenu) myMenu.hidePopup();
  
  // clear aData
  aData = null;
}	

/*
 * Go to mail url if user clicks alert window.  Since user can only left click an alert, 
 * this will only open in a new tab or window based off the single click (or double click
 * if single click is not set or set to same tab) preference. This will never overwrite 
 * the current tab since user might not want to do so.
 */
ym_notifier.prototype.clickAlert = function() {
  //If user clicks alert on mail in new tab
  try {
    var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].
               getService(Components.interfaces.nsIWindowMediator);
    var top = wm.getMostRecentWindow("navigator:browser");
    if (!top)
      this.gotoYahooMail(this.prefs.CONST_NEWWIN, null);
    else {
      var option = this.prefs.getPref(this.prefs.PREF_CLICK);
      if ((option == this.prefs.CONST_NOTHING) || (option == this.prefs.CONST_SAMETAB))
        option = this.prefs.getPref(this.prefs.PREF_DOUBLECLICK);
      if ((option == this.prefs.CONST_NOTHING) || (option == this.prefs.CONST_SAMETAB))
        option = this.prefs.CONST_FOCUSTAB;
      this.gotoYahooMail(option, top.document.getElementById("content"));
    }
  }
  catch(ex) {
    // There are no browser windows open and therefore no component services available (no preferences)
    // so open a new window and go to http://mail.yahoo.com
    window.openDialog("chrome://browser/content/browser.xul", "_blank",
                                "chrome,all,dialog=no", this.mailURL, null, null);
  }
}

/* 
 * handles single, double and middle button mouse clicks
 */
ym_notifier.prototype.handleClick = function(event) {
  var prefs = gYm_notifier.prefs;
	
  // ignore clicks with ctrl or alt key pressed
  if ((event.ctrlKey)||(event.altKey)) return false;

  // for our purposes a double click is 2 single clicks
  if (event.type == "click") {
    var eShiftKey = ((!(event.ctrlKey)||(event.metaKey)) && (!event.altKey) && (event.shiftKey));
    var midMouse = ((event.button == 1) || ((event.button == 0) && (event.shiftKey)))
               
    // if not a left click, middle mouse click or shift click return
    if ((event.button != 0) && !midMouse) return false;
         
    if (midMouse) { 
      this.gotoYahooMail(prefs.getPref(prefs.PREF_MIDDLECLICK), document.getElementById("content"));
    }
    else {
      var dcTime = prefs.getPref(prefs.PREF_DOUBLECLICK_TIME);

      // If we've just had a click then this is a double click
      if (this.mouseId) {
        clearTimeout( this.mouseId );   // Clear pending Click  
        this.mouseId = null;
        this.gotoYahooMail(prefs.getPref(prefs.PREF_DOUBLECLICK), document.getElementById("content"));
      }
      // first click, might be start of double click so wait a few ms before processing
      else {
        var d = new Date();
        var now = d.getTime();

        this.mouseEvtTime = now;
        this.mouseId = setTimeout("gYm_notifier.doClick(gYm_notifier.prefs.getPref(gYm_notifier.prefs.PREF_CLICK));", dcTime);
      }
    }
  }
  return false;
}

/* 
 * this handles single clicks that aren't actually double clicks
 */
ym_notifier.prototype.doClick = function(clickPref) {
  this.mouseId = null;
  this.gotoYahooMail(clickPref, document.getElementById("content"));
  return false;
}
   	

/* 
 * opens Yahoo! Mail based off of user clicking preferences
 */
ym_notifier.prototype.gotoYahooMail = function(option, tBrowser) {
	// read stored mailurl or use default if none stored
  var prefs = this.prefs;
  var mailURL = this.mailURL;
  var userInfo = this.prefs.loadUser("", false);
  if (userInfo && userInfo.mailURL) mailURL = userInfo.mailURL;
  if (userInfo) delete userInfo;
  
  switch (option) {
    case prefs.CONST_NOTHING:
      return;
      break;
   	        
    case prefs.CONST_SAMETAB:
      tBrowser.loadURI(mailURL);
      break;
   	           
    case prefs.CONST_NEWTAB:
      tBrowser.addTab(mailURL);
      break;
   	           
    case prefs.CONST_FOCUSTAB:
      var tab = tBrowser.addTab(mailURL);
      tBrowser.selectedTab = tab;
      break;
   	           
    case prefs.CONST_NEWWIN:
      window.open(mailURL, "", "");
      break;
  }

  // set icons and text to no new mail
  this.alertUser(0);
}

/*
 * Open the preferences window
 */ 
ym_notifier.prototype.openPrefWindow = function() {
  window.openDialog('chrome://ym_notifier/content/ym_prefs.xul', '', 
                    'close=no, dialog, chrome, modal, centerscreen');
}

/*
 * Open the login window
 */ 
ym_notifier.prototype.openLoginWindow = function() {
  window.openDialog('chrome://ym_notifier/content/ym_login.xul', '', 
                    'close=no, dialog, chrome, modal, centerscreen');
}

/*
 * Get the localized strings
 */
ym_notifier.prototype.getString = function(stringName) {
  var strings = document.getElementById("ym-notifierStrings");
  return strings.getString(stringName);
}

/*
 * Find number of messages in string
 */
ym_notifier.prototype.parseNumbFromString = function(aString) {
  var nbMessages = 0;
  var position = -1;
  for (i = 0; i<aString.length; i++)
  {
    if ((aString.charAt(i) >= '0') && (aString.charAt(i) <= '9')) {
      position = i;
      break;
    }
  }
  if (position != -1) 
    nbMessages = parseInt(aString.substring(position,aString.length),10);

  this.ym_log('Number of messages: ' + nbMessages + '\n');

  // Make sure that it is a number
  var objRegExp  = /(^-?\d\d*$)/;
  if (!objRegExp.test(nbMessages)) nbMessages = 0;

  return nbMessages
}

/**
 * Utility to create an error message in the log without throwing an error.
 */
ym_notifier.prototype.ym_logError = function(e, force) {
  try { 
    if (force || this.prefs.getPref(this.prefs.PREF_LOG_ENABLED)) {
      var consoleService = Components.classes['@mozilla.org/consoleservice;1'].
                           getService(Components.interfaces.nsIConsoleService);

      var consoleError = Components.classes['@mozilla.org/scripterror;1'].
                         createInstance(Components.interfaces.nsIScriptError);

      consoleError.init(e.message, e.fileName, e.lineNumber, e.lineNumber,
                        e.columnNumber, 0, null);

      consoleService.logMessage(consoleError);
    }
  }
  catch (ex) {
    dump (ex)
  }
}

/*
 * Log info messages
 */
ym_notifier.prototype.ym_log = function(aMessage, force) {
  try {
    if (force || this.prefs.getPref(this.prefs.PREF_LOG_ENABLED)) {
      var consoleService = Components.classes['@mozilla.org/consoleservice;1'].
                           getService(Components.interfaces.nsIConsoleService);
      consoleService.logStringMessage("ym_notifier: " + aMessage);
    }
  }
  catch (ex) { 
    dump(ex); 
  }
}

/*
 * Uninstall procedure
 */
ym_notifier.prototype.uninstall = function() {
  var confirmUninstall = confirm(this.getString("confirmuninstall"));
	
  // user choose to cancel
  if (!confirmUninstall) return; // do nothing further
	
  // disable the "Uninstall" menuitem
  document.getElementById("ym-notifier-context-menu-uninstall").setAttribute("disabled", true); 
	
  var oExtUninstall = new ExtensionUninstall("urn:mozilla:package:ym_notifier");
  var success = oExtUninstall.uninstall();
  this.uninstalled(oExtUninstall);
}

/*
 * Output Result of Uninstall
 */
ym_notifier.prototype.uninstalled = function(oExtUninstall) {
  var message;
	
  if(oExtUninstall != null && oExtUninstall.hasError())
    message = this.getString("uninstallerror");
  else if(oExtUninstall != null && oExtUninstall.hasWarning())
    message = this.getString("uninstallwarning");
  else
    message = this.getString("uninstallsuccess");
	
  var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].
                getService(Components.interfaces.nsIPromptService);
  var flags = prompts.BUTTON_TITLE_IS_STRING * prompts.BUTTON_POS_0 + 
              prompts.BUTTON_TITLE_IS_STRING * prompts.BUTTON_POS_1;
  var check = {value: false};
  var button = prompts.confirmEx(window, this.getString("uninstalldone"), message, flags, 
                                 this.getString("closebutton"), this.getString("viewlogbuton"), null, null, check);
  if (button) oExtUninstall.showLogDialog(true, self);
}

var gYm_notifier = new ym_notifier();
   
// only add listeners to browser windows
if (document.getElementById("content"))
{
   window.addEventListener('load', gYm_notifier.initialize, false);
   window.addEventListener('unload', gYm_notifier.uninitialize, false);
}